"use strict";

var _interopRequireDefault = require("babel-runtime/helpers/interop-require-default")["default"];

exports.__esModule = true;
exports["default"] = get;

var _pathExists = require("path-exists");

var _pathExists2 = _interopRequireDefault(_pathExists);

var _trimRight = require("trim-right");

var _trimRight2 = _interopRequireDefault(_trimRight);

var _tryResolve = require("try-resolve");

var _tryResolve2 = _interopRequireDefault(_tryResolve);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

function humanize(val, noext) {
  if (noext) val = _path2["default"].basename(val, _path2["default"].extname(val));
  return val.replace(/-/g, " ");
}

function get(entryLoc) {
  var suites = [];

  var rootOpts = {};
  var rootOptsLoc = _tryResolve2["default"](entryLoc + "/options");
  if (rootOptsLoc) rootOpts = require(rootOptsLoc);

  _lodash2["default"].each(_fs2["default"].readdirSync(entryLoc), function (suiteName) {
    if (suiteName[0] === ".") return;

    var suite = {
      options: _lodash2["default"].clone(rootOpts),
      tests: [],
      title: humanize(suiteName),
      filename: entryLoc + "/" + suiteName
    };
    suites.push(suite);

    var suiteOptsLoc = _tryResolve2["default"](suite.filename + "/options");
    if (suiteOptsLoc) suite.options = require(suiteOptsLoc);

    if (_fs2["default"].statSync(suite.filename).isFile()) {
      push(suiteName, suite.filename);
    } else {
      _lodash2["default"].each(_fs2["default"].readdirSync(suite.filename), function (taskName) {
        var taskDir = suite.filename + "/" + taskName;
        push(taskName, taskDir);
      });
    }

    function push(taskName, taskDir) {
      // tracuer error tests
      if (taskName.indexOf("Error_") >= 0) return;

      var actualLocAlias = suiteName + "/" + taskName + "/actual.js";
      var expectLocAlias = suiteName + "/" + taskName + "/expected.js";
      var execLocAlias = suiteName + "/" + taskName + "/exec.js";

      var actualLoc = taskDir + "/actual.js";
      var expectLoc = taskDir + "/expected.js";
      var execLoc = taskDir + "/exec.js";

      if (_tryResolve2["default"].relative(expectLoc + "on")) {
        expectLoc += "on";
        expectLocAlias += "on";
      }

      if (_fs2["default"].statSync(taskDir).isFile()) {
        var ext = _path2["default"].extname(taskDir);
        if (ext !== ".js" && ext !== ".module.js") return;

        execLoc = taskDir;
      }

      var taskOpts = _lodash2["default"].merge({
        filenameRelative: expectLocAlias,
        sourceFileName: actualLocAlias,
        sourceMapName: expectLocAlias
      }, _lodash2["default"].cloneDeep(suite.options));

      var taskOptsLoc = _tryResolve2["default"](taskDir + "/options");
      if (taskOptsLoc) _lodash2["default"].merge(taskOpts, require(taskOptsLoc));

      var test = {
        title: humanize(taskName, true),
        disabled: taskName[0] === ".",
        options: taskOpts,
        exec: {
          loc: execLoc,
          code: readFile(execLoc),
          filename: execLocAlias
        },
        actual: {
          loc: actualLoc,
          code: readFile(actualLoc),
          filename: actualLocAlias
        },
        expect: {
          loc: expectLoc,
          code: readFile(expectLoc),
          filename: expectLocAlias
        }
      };

      // traceur checks

      var shouldSkip = function shouldSkip(code) {
        return code.indexOf("// Error:") >= 0 || code.indexOf("// Skip.") >= 0;
      };

      if (shouldSkip(test.actual.code) || shouldSkip(test.exec.code)) {
        return;
      } else if (test.exec.code.indexOf("// Async.") >= 0) {
        //test.options.asyncExec = true;
        return;
      }

      suite.tests.push(test);

      var sourceMappingsLoc = taskDir + "/source-mappings.json";
      if (_pathExists2["default"].sync(sourceMappingsLoc)) {
        test.sourceMappings = JSON.parse(readFile(sourceMappingsLoc));
      }

      var sourceMapLoc = taskDir + "/source-map.json";
      if (_pathExists2["default"].sync(sourceMapLoc)) {
        test.sourceMap = JSON.parse(readFile(sourceMapLoc));
      }
    }
  });

  return suites;
}

function readFile(filename) {
  if (_pathExists2["default"].sync(filename)) {
    var file = _trimRight2["default"](_fs2["default"].readFileSync(filename, "utf8"));
    file = file.replace(/\r\n/g, "\n");
    return file;
  } else {
    return "";
  }
}
module.exports = exports["default"];